@@ -248,17 +248,22 @@ class DDParser(object):
 
     Args:
     use_cuda: BOOL, 是否使用gpu
-    tree: BOOL， 是否返回树结构
-    prob: BOOL， 是否返回弧的概率
+    tree: BOOL, 是否返回树结构
+    prob: BOOL, 是否返回弧的概率
     use_pos: BOOL, 是否返回词性标签(仅parse函数生效)
     model_files_path: str, 模型地址, 为None时下载默认模型
+    buckets: BOOL, 是否对样本分桶. 若buckets=True，则会对inputs按长度分桶，处理长度不均匀的输入速度更新快，default=False
+    batch_size: INT, 批尺寸, 当buckets为False时，每个batch大小均等于batch_size; 当buckets为True时，每个batch的大小约为'batch_size / 当前桶句子的平均长度'。
+                当default=None时，分桶batch_size默认等于1000，不分桶默认等于50。
     """
     def __init__(self,
                  use_cuda=False,
                  tree=True,
                  prob=False,
                  use_pos=False,
-                 model_files_path=None):
+                 model_files_path=None,
+                 buckets=False,
+                 batch_size=None):
         if model_files_path is None:
             model_files_path = self._get_abs_path('./model_files/baidu')
             if not os.path.exists(model_files_path):
@@ -280,6 +285,8 @@ class DDParser(object):
             args.append("--tree")
         if prob:
             args.append("--prob")
+        if batch_size:
+            args.append(f"--batch_size={batch_size}")
 
         args = ArgConfig(args)
         # Don't instantiate the log handle
@@ -290,11 +297,16 @@ class DDParser(object):
         self.model = load(self.args.model_path)
         self.lac = None
         self.use_pos = use_pos
-
+        # buckets=None if not buckets else defaults
+        if not buckets:
+            self.args.buckets = None
         if args.prob:
             self.env.fields = self.env.fields._replace(PHEAD=Field('prob'))
         if self.use_pos:
             self.env.fields = self.env.fields._replace(CPOS=Field('postag'))
+        # set default batch size if batch_size is None and not buckets
+        if batch_size is None and not buckets:
+            self.args.batch_size = 50
 
     def parse(self, inputs):
         """
@@ -324,29 +336,47 @@ class DDParser(object):
         'head': [2, 0, 5, 5, 2], 'deprel': ['SBV', 'HED', 'ATT', 'ATT', 'VOB'], 'prob': [1.0, 1.0, 1.0, 1.0, 1.0]}]
         """
         if not self.lac:
-            self.lac = LAC.LAC(mode='lac' if self.use_pos else "seg")
+            self.lac = LAC.LAC(mode='lac' if self.use_pos else "seg",
+                               use_cuda=self.args.use_cuda)
         if not inputs:
             return
         if isinstance(inputs, str):
             inputs = [inputs]
         if all([isinstance(i, str) and i for i in inputs]):
-            lac_results = self.lac.run(inputs)
+            lac_results = []
+            position = 0
+            while position < len(inputs):
+                lac_results += self.lac.run(inputs[position:position +
+                                                   self.args.batch_size])
+                position += self.args.batch_size
             predicts = Corpus.load_lac_results(lac_results, self.env.fields)
         else:
             logging.warning("please check the foramt of your inputs.")
             return
-        dataset = TextDataset(predicts, [self.env.WORD, self.env.FEAT])
+        dataset = TextDataset(predicts, [self.env.WORD, self.env.FEAT],
+                              self.args.buckets)
         # set the data loader
-        dataset.loader = batchify(dataset,
-                                  self.args.batch_size,
-                                  use_multiprocess=False,
-                                  sequential_sampler=True)
+
+        dataset.loader = batchify(
+            dataset,
+            self.args.batch_size,
+            use_multiprocess=False,
+            sequential_sampler=True if not self.args.buckets else False)
         pred_arcs, pred_rels, pred_probs = epoch_predict(
             self.env, self.args, self.model, dataset.loader)
-        predicts.head = pred_arcs
-        predicts.deprel = pred_rels
+
+        if self.args.buckets:
+            indices = np.argsort(
+                np.array([
+                    i for bucket in dataset.buckets.values() for i in bucket
+                ]))
+        else:
+            indices = range(len(pred_arcs))
+        predicts.head = [pred_arcs[i] for i in indices]
+        predicts.deprel = [pred_rels[i] for i in indices]
         if self.args.prob:
-            predicts.prob = pred_probs
+            predicts.prob = [pred_probs[i] for i in indices]
+
         outputs = predicts.get_result()
         return outputs
 
@@ -381,18 +411,29 @@ class DDParser(object):
         else:
             logging.warning("please check the foramt of your inputs.")
             return
-        dataset = TextDataset(predicts, [self.env.WORD, self.env.FEAT])
+        dataset = TextDataset(predicts, [self.env.WORD, self.env.FEAT],
+                              self.args.buckets)
         # set the data loader
-        dataset.loader = batchify(dataset,
-                                  self.args.batch_size,
-                                  use_multiprocess=False,
-                                  sequential_sampler=True)
+        dataset.loader = batchify(
+            dataset,
+            self.args.batch_size,
+            use_multiprocess=False,
+            sequential_sampler=True if not self.args.buckets else False)
         pred_arcs, pred_rels, pred_probs = epoch_predict(
             self.env, self.args, self.model, dataset.loader)
-        predicts.head = pred_arcs
-        predicts.deprel = pred_rels
+
+        if self.args.buckets:
+            indices = np.argsort(
+                np.array([
+                    i for bucket in dataset.buckets.values() for i in bucket
+                ]))
+        else:
+            indices = range(len(pred_arcs))
+        predicts.head = [pred_arcs[i] for i in indices]
+        predicts.deprel = [pred_rels[i] for i in indices]
         if self.args.prob:
-            predicts.prob = pred_probs
+            predicts.prob = [pred_probs[i] for i in indices]
+
         outputs = predicts.get_result()
         if outputs[0].get("postag", None):
             for output in outputs:
